@@ -5,6 +5,7 @@ from typing import Any, TypeVar
 from sqlalchemy.orm.attributes import InstrumentedAttribute
 from sqlalchemy.sql.elements import ColumnElement
 from sqlalchemy.sql.operators import OperatorType
+from sqlalchemy.sql.selectable import Select
 
 # ruff: noqa: F401
 if sys.version_info >= (3, 11):
@@ -14,6 +15,8 @@ else:
 
 T = TypeVar('T')
 
+Query = Select[tuple[T, ...]]
+
 EagerLoadPath = (
     InstrumentedAttribute[Any] | tuple[InstrumentedAttribute[Any], bool]
 )
